@@ -35,10 +35,13 @@ class Crossbar():
         if len(shape) == 4: # memtorch.mn.Conv2d
             self.rows = shape[0]
             self.columns = shape[1] * shape[2] * shape[3]
+        elif len(shape) == 3: # memtorch.mn.Conv1d
+            self.rows = shape[0]
+            self.columns = shape[1] * shape[2]
         elif len(shape) == 2: # memtorch.mn.Linear
             self.columns, self.rows = shape
         else:
-            raise('Unsupported crossbar shape.')
+            raise Exception('Unsupported crossbar shape.')
 
         self.rows = int(self.rows)
         self.columns = int(self.columns)
@@ -84,7 +87,7 @@ class Crossbar():
             Programming routine (method) to use.
         """
         if transistor:
-            if len(conductance_matrix.shape) == 4: # memtorch.mn.Conv2d
+            if len(conductance_matrix.shape) == 4 or len(conductance_matrix.shape) == 3: # memtorch.mn.Conv1d and memtorch.mn.Conv2d
                 self.conductance_matrix = conductance_matrix.reshape(self.rows, self.columns)
             elif len(conductance_matrix.shape) == 2: # memtorch.mn.Linear
                 conductance_matrix = conductance_matrix.T.clone().detach().to(self.device)
@@ -105,7 +108,7 @@ class Crossbar():
             self.update(from_devices=True)
 
 
-def init_crossbar(weights, memristor_model, memristor_model_params, transistor, mapping_routine, programming_routine, scheme=Scheme.DoubleColumn):
+def init_crossbar(weights, memristor_model, memristor_model_params, transistor, mapping_routine, programming_routine, p_l=None, scheme=Scheme.DoubleColumn):
     """Method to initialise and construct memristive crossbars.
 
     Parameters
@@ -122,6 +125,8 @@ def init_crossbar(weights, memristor_model, memristor_model_params, transistor,
         Mapping routine to use.
     programming_routine : function
         Programming routine to use.
+    p_l: float
+        If not None, the proportion of weights to retain.
     scheme : memtorch.bh.Scheme
         Scheme enum.
 
@@ -141,7 +146,8 @@ def init_crossbar(weights, memristor_model, memristor_model_params, transistor,
         pos_conductance_matrix, neg_conductance_matrix = mapping_routine(weights_,
                                                                          reference_memristor_model.r_on,
                                                                          reference_memristor_model.r_off,
-                                                                         scheme)
+                                                                         scheme=scheme,
+                                                                         p_l=p_l)
 
         crossbars[0].write_conductance_matrix(pos_conductance_matrix, transistor=transistor, programming_routine=programming_routine)
         crossbars[1].write_conductance_matrix(neg_conductance_matrix, transistor=transistor, programming_routine=programming_routine)
@@ -153,7 +159,8 @@ def init_crossbar(weights, memristor_model, memristor_model_params, transistor,
         conductance_matrix = mapping_routine(weights_,
                                              reference_memristor_model.r_on,
                                              reference_memristor_model.r_off,
-                                             scheme)
+                                             scheme=scheme,
+                                             p_l=p_l)
         crossbars[0].write_conductance_matrix(conductance_matrix, transistor=transistor, programming_routine=programming_routine)
         g_m = ((1 / reference_memristor_model.r_on) + (1 / reference_memristor_model.r_off)) / 2
         def out(crossbars, operation, *args):
@@ -164,7 +171,7 @@ def init_crossbar(weights, memristor_model, memristor_model_params, transistor,
 
     return crossbars, out
 
-def simulate(input, devices, parallelize=False, nl=True):
+def simulate_matmul(input, devices, parallelize=False, nl=True):
     """Method to simulate non-linear IV device characterisitcs for a 2-D crossbar architecture given scaled inputs.
 
     Parameters
