@@ -27,12 +27,12 @@ class Conv2d(nn.Conv2d):
     programming_routine : function
         Programming routine to use.
     p_l: float
-        Proportion of weights to retain.
+        If not None, the proportion of weights to retain.
     scheme : memtorch.bh.Scheme
         Weight representation scheme.
     """
 
-    def __init__(self, convolutional_layer, memristor_model, memristor_model_params, mapping_routine=naive_map, transistor=False, programming_routine=None, p_l=1.0, scheme=memtorch.bh.Scheme.DoubleColumn, *args, **kwargs):
+    def __init__(self, convolutional_layer, memristor_model, memristor_model_params, mapping_routine=naive_map, transistor=False, programming_routine=None, p_l=None, scheme=memtorch.bh.Scheme.DoubleColumn, *args, **kwargs):
         assert isinstance(convolutional_layer, nn.Conv2d), 'convolutional_layer is not an instance of nn.Conv2d.'
         self.device = torch.device('cpu' if 'cpu' in memtorch.__version__ else 'cuda')
         super(Conv2d, self).__init__(convolutional_layer.in_channels, convolutional_layer.out_channels, convolutional_layer.kernel_size, **kwargs)
@@ -71,45 +71,33 @@ class Conv2d(nn.Conv2d):
             torch.Tensor
                 Output tensor.
         """
-        output_dim = int((input.shape[2] - self.kernel_size[0] + 2 * self.padding[0]) / self.stride[0]) + 1
-        out = torch.zeros((input.shape[0], self.out_channels, output_dim, output_dim)).to(self.device)
-        for batch in range(input.shape[0]):
-            unfolded_batch_input = torch.nn.functional.unfold(input[batch, :, :, :].unsqueeze(0), kernel_size=self.kernel_size, padding=self.padding)
-            if hasattr(self, 'non_linear'):
-                unfolded_batch_input = convert_range(unfolded_batch_input, unfolded_batch_input.min(), unfolded_batch_input.max(), -1, 1).squeeze(0)
-                unfolded_batch_input = unfolded_batch_input.transpose(1, 0).cpu().detach().numpy()
-                if hasattr(self, 'simulate'):
-                    out_ = torch.tensor(self.transform_output(self.crossbar_operation(self.crossbars, lambda crossbar, input: simulate_matmul(input, crossbar.devices.transpose(1, 0), nl=False), unfolded_batch_input))).to(self.device)
+        if self.forward_legacy_enabled:
+            return torch.nn.functional.conv2d(input.to(self.device), self.weight, bias=self.bias, stride=self.stride, padding=self.padding)
+        else:
+            output_dim = int((input.shape[2] - self.kernel_size[0] + 2 * self.padding[0]) / self.stride[0]) + 1
+            out = torch.zeros((input.shape[0], self.out_channels, output_dim, output_dim)).to(self.device)
+            for batch in range(input.shape[0]):
+                unfolded_batch_input = torch.nn.functional.unfold(input[batch, :, :, :].unsqueeze(0), kernel_size=self.kernel_size, padding=self.padding)
+                if hasattr(self, 'non_linear'):
+                    unfolded_batch_input = convert_range(unfolded_batch_input, unfolded_batch_input.min(), unfolded_batch_input.max(), -1, 1).squeeze(0)
+                    unfolded_batch_input = unfolded_batch_input.transpose(1, 0).cpu().detach().numpy()
+                    if hasattr(self, 'simulate'):
+                        out_ = torch.tensor(self.transform_output(self.crossbar_operation(self.crossbars, lambda crossbar, input: simulate_matmul(input, crossbar.devices.transpose(1, 0), nl=False), unfolded_batch_input))).to(self.device)
+                    else:
+                        out_ = torch.tensor(self.transform_output(self.crossbar_operation(self.crossbars, lambda crossbar, input: simulate_matmul(input, crossbar.devices.transpose(1, 0), nl=True), unfolded_batch_input))).to(self.device)
                 else:
-                    out_ = torch.tensor(self.transform_output(self.crossbar_operation(self.crossbars, lambda crossbar, input: simulate_matmul(input, crossbar.devices.transpose(1, 0), nl=True), unfolded_batch_input))).to(self.device)
-            else:
-                out_ = self.transform_output(torch.matmul(self.crossbar_operation(self.crossbars, lambda crossbar: crossbar.conductance_matrix), unfolded_batch_input))
+                    out_ = self.transform_output(torch.matmul(self.crossbar_operation(self.crossbars, lambda crossbar: crossbar.conductance_matrix), unfolded_batch_input))
 
-            if not self.bias is None:
-                out_ += self.bias.view(-1, 1).expand_as(out_)
+                if not self.bias is None:
+                    out_ += self.bias.view(-1, 1).expand_as(out_)
 
-            out[batch] = out_.view(size=(1, self.out_channels, output_dim, output_dim))
+                out[batch] = out_.view(size=(1, self.out_channels, output_dim, output_dim))
 
-        return out
+            return out
 
-    def forward_legacy(self, input):
-        """Legacy method to perform forward propagations.
-
-            Parameters
-            ----------
-            input : torch.Tensor
-                Input tensor.
-
-            Returns
-            -------
-            torch.Tensor
-                Output tensor.
-        """
-        return torch.nn.functional.conv2d(input.to(self.device), self.weight, bias=self.bias, stride=self.stride, padding=self.padding)
-
-    def tune(self):
+    def tune(self, input_batch_size=8, input_shape=32):
         """Tuning method."""
-        self.transform_output = naive_tune(self, (8, self.in_channels, 32, 32))
+        self.transform_output = naive_tune(self, (input_batch_size, self.in_channels, input_shape, input_shape))
 
     def __str__(self):
         return "bh.Conv2d(in_channels=%d, out_channels=%d, kernel_size=(%d, %d), stride=(%d,%d), padding=(%d,%d))" % (self.in_channels, self.out_channels, self.kernel_size[0], self.kernel_size[1], self.stride[0], self.stride[1], self.padding[0], self.padding[1])
